@@ -5,27 +5,45 @@
 #  See http://subversion.tigris.org for more information.
 #
 # ====================================================================
-# Copyright (c) 2001, 2006 CollabNet.  All rights reserved.
+#    Licensed to the Apache Software Foundation (ASF) under one
+#    or more contributor license agreements.  See the NOTICE file
+#    distributed with this work for additional information
+#    regarding copyright ownership.  The ASF licenses this file
+#    to you under the Apache License, Version 2.0 (the
+#    "License"); you may not use this file except in compliance
+#    with the License.  You may obtain a copy of the License at
 #
-# This software is licensed as described in the file COPYING, which
-# you should have received as part of this distribution.  The terms
-# are also available at http://subversion.tigris.org/license-1.html.
-# If newer versions of this license are posted there, you may use a
-# newer version instead, at your option.
+#      http://www.apache.org/licenses/LICENSE-2.0
 #
+#    Unless required by applicable law or agreed to in writing,
+#    software distributed under the License is distributed on an
+#    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+#    KIND, either express or implied.  See the License for the
+#    specific language governing permissions and limitations
+#    under the License.
 ######################################################################
 
 import re
 import os
 import sys
+if sys.version_info[0] >= 3:
+  # Python >=3.0
+  from io import StringIO
+else:
+  # Python <3.0
+  from cStringIO import StringIO
+from xml.dom.minidom import parseString
+import base64
+import logging
 
-import main  # the general svntest routines in this module.
-from svntest import Failure
+import svntest
+
+logger = logging.getLogger()
 
 # Tree Exceptions.
 
 # All tree exceptions should inherit from SVNTreeError
-class SVNTreeError(Failure):
+class SVNTreeError(svntest.Failure):
   "Exception raised if you screw up in the tree module."
   pass
 
@@ -33,10 +51,6 @@ class SVNTreeUnequal(SVNTreeError):
   "Exception raised if two trees are unequal."
   pass
 
-class SVNTreeIsNotDirectory(SVNTreeError):
-  "Exception raised if get_child is passed a file."
-  pass
-
 class SVNTypeMismatch(SVNTreeError):
   "Exception raised if one node is file and other is dir"
   pass
@@ -172,7 +186,7 @@ class SVNTreeNode:
         a.contents = newchild.contents
         a.props = newchild.props
         a.atts = newchild.atts
-        a.path = os.path.join (self.path, newchild.name)
+        a.path = os.path.join(self.path, newchild.name)
       else:
         # try to add dangling children to your matching node
         for i in newchild.children:
@@ -183,19 +197,19 @@ class SVNTreeNode:
 
 
   def pprint(self, stream = sys.stdout):
-    "Pretty-print the meta data for this node."
-    print >> stream, " * Node name:  ", self.name
-    print >> stream, "    Path:      ", self.path
+    "Pretty-print the meta data for this node to STREAM."
+    stream.write(" * Node name:   %s\n" % self.name)
+    stream.write("    Path:       %s\n" % self.path)
     mime_type = self.props.get("svn:mime-type")
     if not mime_type or mime_type.startswith("text/"):
       if self.children is not None:
-        print >> stream, "    Contents:   N/A (node is a directory)"
+        stream.write("    Contents:   N/A (node is a directory)\n")
       else:
-        print >> stream, "    Contents:  ", self.contents
+        stream.write("    Contents:   %s\n" % self.contents)
     else:
-      print >> stream, "    Contents:   %d bytes (binary)" % len(self.contents)
-    print >> stream, "    Properties:", self.props
-    print >> stream, "    Attributes:", self.atts
+      stream.write("    Contents:   %d bytes (binary)\n" % len(self.contents))
+    stream.write("    Properties: %s\n" % self.props)
+    stream.write("    Attributes: %s\n" % self.atts)
     ### FIXME: I'd like to be able to tell the difference between
     ### self.children is None (file) and self.children == [] (empty
     ### directory), but it seems that most places that construct
@@ -203,17 +217,184 @@ class SVNTreeNode:
     ###
     ### See issue #1611 about this problem.  -kfogel
     if self.children is not None:
-      print >> stream, "    Children:  ", len(self.children)
+      stream.write("    Children:   %s\n" % len(self.children))
     else:
-      print >> stream, "    Children:   N/A (node is a file)"
+      stream.write("    Children:  None (node is probably a file)\n")
+    stream.flush()
+
+  def get_printable_path(self):
+    """Remove some occurrences of root_node_name = "__SVN_ROOT_NODE",
+    it is in the way when matching for a subtree, and looks bad."""
+    path = self.path
+    if path.startswith(root_node_name + os.sep):
+      path = path[len(root_node_name + os.sep):]
+    return path
+
+  def print_script(self, stream = sys.stdout, subtree = "", prepend="\n  ",
+                   drop_empties = True):
+    """Python-script-print the meta data for this node to STREAM.
+    Print only those nodes whose path string starts with the string SUBTREE,
+    and print only the part of the path string that remains after SUBTREE.
+    PREPEND is a string prepended to each node printout (does the line
+    feed if desired, don't include a comma in PREPEND).
+    If DROP_EMPTIES is true, all dir nodes that have no data set in them
+    (no props, no atts) and that have children (so they are included
+    implicitly anyway) are not printed.
+    Return 1 if this node was printed, 0 otherwise (added up by
+    dump_tree_script())"""
+
+    # figure out if this node would be obsolete to print.
+    if drop_empties and len(self.props) < 1 and len(self.atts) < 1 and \
+       self.contents is None and self.children is not None:
+      return 0
+
+    path = self.get_printable_path()
+
+    # remove the subtree path, skip this node if necessary.
+    if path.startswith(subtree):
+      path = path[len(subtree):]
+    elif path + os.sep == subtree:
+      # Many callers set subtree to 'some-path' + os.sep. Don't skip the
+      # root node in that case.
+      path = ''
+    else:
+      return 0
+
+    if path.startswith(os.sep):
+      path = path[1:]
+
+    line = prepend
+    line += "%-20s: Item(" % ("'%s'" % path.replace(os.sep, '/'))
+    comma = False
+
+    mime_type = self.props.get("svn:mime-type")
+    if not mime_type or mime_type.startswith("text/"):
+      if self.contents is not None:
+        # Escape some characters for nicer script and readability.
+        # (This is error output. I guess speed is no consideration here.)
+        line += "contents=\"%s\"" % (self.contents
+                                     .replace('\n','\\n')
+                                     .replace('"','\\"')
+                                     .replace('\r','\\r')
+                                     .replace('\t','\\t'))
+        comma = True
+    else:
+      line += 'content is binary data'
+      comma = True
+
+    if self.props:
+      if comma:
+        line += ", "
+      line += "props={"
+      comma = False
+
+      for name in self.props:
+        if comma:
+          line += ", "
+        line += "'%s':'%s'" % (name, self.props[name])
+        comma = True
+
+      line += "}"
+      comma = True
+
+    for name in self.atts:
+      if comma:
+        line += ", "
+      line += "%s='%s'" % (name, self.atts[name])
+      comma = True
+
+    line += "),"
+    stream.write("%s" % line)
+    stream.flush()
+    return 1
+
 
   def __str__(self):
-    import StringIO
-    s = StringIO.StringIO()
+    s = StringIO()
     self.pprint(s)
     return s.getvalue()
 
 
+  def __cmp__(self, other):
+    """Define a simple ordering of two nodes without regard to their full
+    path (i.e. position in the tree). This can be used for sorting the
+    children within a directory."""
+    return cmp(self.name, other.name)
+
+  def as_state(self, prefix=None):
+    """Return an svntest.wc.State instance that is equivalent to this tree."""
+    root = self
+    if self.path == root_node_name:
+      assert prefix is None
+      wc_dir = ''
+      while True:
+        if root is not self:  # don't prepend ROOT_NODE_NAME
+          wc_dir = os.path.join(wc_dir, root.name)
+        if root.contents or root.props or root.atts:
+          break
+        if not root.children or len(root.children) != 1:
+          break
+        root = root.children[0]
+      state = svntest.wc.State(wc_dir, { })
+      if root.contents or root.props or root.atts:
+        state.add({'': root.as_item()})
+      prefix = wc_dir
+    else:
+      assert prefix is not None
+
+      path = self.path
+      if path.startswith(root_node_name):
+        path = path[len(root_node_name)+1:]
+      # prefix should only be set on a recursion, which means a child,
+      # which means this path better not be the same as the prefix.
+      assert path != prefix, 'not processing a child of the root'
+      l = len(prefix)
+      if l > 0:
+        assert path[:l] == prefix, \
+            '"%s" is not a prefix of "%s"' % (prefix, path)
+        # return the portion after the separator
+        path = path[l+1:].replace(os.sep, '/')
+
+      state = svntest.wc.State('', {
+          path: self.as_item()
+          })
+
+    if root.children:
+      for child in root.children:
+        state.add_state('', child.as_state(prefix))
+
+    return state
+
+  def as_item(self):
+    return svntest.wc.StateItem(self.contents,
+                                self.props,
+                                self.atts.get('status'),
+                                self.atts.get('verb'),
+                                self.atts.get('wc_rev'),
+                                self.atts.get('locked'),
+                                self.atts.get('copied'),
+                                self.atts.get('switched'),
+                                self.atts.get('writelocked'),
+                                self.atts.get('treeconflict'))
+
+  def recurse(self, function):
+    results = []
+    results += [ function(self) ]
+    if self.children:
+      for child in self.children:
+        results += child.recurse(function)
+    return results
+
+  def find_node(self, path):
+    if self.get_printable_path() == path:
+      return self
+    if self.children:
+      for child in self.children:
+        result = child.find_node(path)
+        if result:
+          return result
+    return None
+
 # reserved name of the root of the tree
 root_node_name = "__SVN_ROOT_NODE"
 
@@ -239,30 +420,45 @@ def add_elements_as_path(top_node, element_list):
     prev_node = new_node
 
 
-# Sorting function -- sort 2 nodes by their names.
-def node_is_greater(a, b):
-  "Sort the names of two nodes."
-  # Interal use only
-  if a.name == b.name:
-    return 0
-  if a.name > b.name:
-    return 1
-  else:
-    return -1
-
-
 # Helper for compare_trees
 def compare_file_nodes(a, b):
-  """Compare two nodes' names, contents, and properties, ignoring
-  children.  Return 0 if the same, 1 otherwise."""
+  """Compare two nodes, A (actual) and B (expected). Compare their names,
+  contents, properties and attributes, ignoring children.  Return 0 if the
+  same, 1 otherwise."""
   if a.name != b.name:
     return 1
   if a.contents != b.contents:
     return 1
   if a.props != b.props:
     return 1
-  if a.atts != b.atts:
+  if a.atts == b.atts:
+    # No fixes necessary
+    return 0
+
+  # Fix a pre-WC-NG assumptions in our testsuite
+  if (b.atts == {'status': 'A ', 'wc_rev': '0'})     \
+     and (a.atts == {'status': 'A ', 'wc_rev': '-'}):
+    return 0
+  return 1
+
+# Helper for compare_trees
+def compare_dir_nodes(a, b):
+  """Compare two nodes, A (actual) and B (expected). Compare their names,
+  properties and attributes, ignoring children.  Return 0 if the
+  same, 1 otherwise."""
+  if a.name != b.name:
     return 1
+  if (a.props != b.props):
+    return 1
+  if (a.atts == b.atts):
+    # No fixes necessary
+    return 0
+
+  # Fix a pre-WC-NG assumptions in our testsuite
+  if (b.atts == {'status': 'A ', 'wc_rev': '0'})     \
+     and (a.atts == {'status': 'A ', 'wc_rev': '-'}):
+    return 0
+  return 1
 
 
 # Internal utility used by most build_tree_from_foo() routines.
@@ -300,7 +496,7 @@ def create_from_path(path, contents=None, props={}, atts={}):
 
   # deposit contents in the very last node.
   node = root_node
-  while 1:
+  while True:
     if node.children is None:
       node.contents = contents
       node.props = props
@@ -311,50 +507,61 @@ def create_from_path(path, contents=None, props={}, atts={}):
   return root_node
 
 
-# helper for handle_dir(), which is a helper for build_tree_from_wc()
-def get_props(path):
-  "Return a hash of props for PATH, using the svn client."
+eol_re = re.compile(r'(\r\n|\r)')
+
+# helper for build_tree_from_wc()
+def get_props(paths):
+  """Return a hash of hashes of props for PATHS, using the svn client. Convert
+     each embedded end-of-line to a single LF character."""
 
   # It's not kosher to look inside .svn/ and try to read the internal
   # property storage format.  Instead, we use 'svn proplist'.  After
   # all, this is the only way the user can retrieve them, so we're
   # respecting the black-box paradigm.
 
-  props = {}
-  output, errput = main.run_svn(1, "proplist", path, "--verbose")
+  files = {}
+  exit_code, output, errput = svntest.main.run_svn(1,
+                                                   "proplist",
+                                                   "--verbose",
+                                                   "--xml",
+                                                   *paths)
 
-  for line in output:
-    if line.startswith('Properties on '):
-      continue
-    if line.startswith('  ') and line.find(' : ') >= 3:
-      name, value = line.split(' : ')
-      name = name[2:]
-      value = value.rstrip("\r\n")
-      props[name] = value
-    else:    # Multi-line property, so re-use the current name.
-      # Keep the line endings consistent with what was done to the first
-      # line by stripping whitespace and then appending a newline.  This
-      # prevents multiline props on Windows that must be stored as UTF8/LF
-      # in the repository (e.g. svn:mergeinfo), say like this:
-      #
-      #   "propname : propvalLine1<LF>propvalLine2<LF>propvalLine3"
-      #
-      # but that print to stdout like this:
-      #
-      #   Properties on 'somepath':<CR><LF>
-      #     propname : propvalLine1<CR><CR><LF>
-      #   propvalLine1<CR><CR><LF>
-      #   propvalLine1<CR><LF>
-      #
-      # from looking like this in the returned PROPS hash:
-      #
-      #   "propname" --> "propvalLine1<LF>propvalLine2<CR><LF>propvalLine3<LF>"
-      props[name] = props[name] + "\n" + line.rstrip("\r\n")
+  output = (line for line in output if not line.startswith('DBG:'))
+  dom = parseString(''.join(output))
+  target_nodes = dom.getElementsByTagName('target')
+  for target_node in target_nodes:
+    filename = target_node.attributes['path'].nodeValue
+    file_props = {}
+    for property_node in target_node.getElementsByTagName('property'):
+      name = property_node.attributes['name'].nodeValue
+      if property_node.hasChildNodes():
+        text_node = property_node.firstChild
+        value = text_node.nodeValue
+      else:
+        value = ''
+      try:
+        encoding = property_node.attributes['encoding'].nodeValue
+        if encoding == 'base64':
+          value = base64.b64decode(value)
+        else:
+          raise Exception("Unknown encoding '%s' for file '%s' property '%s'"
+                          % (encoding, filename, name,))
+      except KeyError:
+        pass
+      # If the property value contained a CR, or if under Windows an
+      # "svn:*" property contains a newline, then the XML output
+      # contains a CR character XML-encoded as '&#13;'.  The XML
+      # parser converts it back into a CR character.  So again convert
+      # all end-of-line variants into a single LF:
+      value = eol_re.sub('\n', value)
+      file_props[name] = value
+    files[filename] = file_props
 
-  return props
+  dom.unlink()
+  return files
 
 
-# helper for handle_dir(), which helps build_tree_from_wc()
+### ridiculous function. callers should do this one line themselves.
 def get_text(path):
   "Return a string with the textual contents of a file at PATH."
 
@@ -362,56 +569,17 @@ def get_text(path):
   if not os.path.isfile(path):
     return None
 
-  fp = open(path, 'r')
-  contents = fp.read()
-  fp.close()
-  return contents
+  return open(path, 'r').read()
 
 
-# main recursive helper for build_tree_from_wc()
-def handle_dir(path, current_parent, load_props, ignore_svn):
-
-  # get a list of all the files
-  all_files = os.listdir(path)
-  files = []
-  dirs = []
-
-  # put dirs and files in their own lists, and remove SVN dirs
-  for f in all_files:
-    f = os.path.join(path, f)
-    if (os.path.isdir(f) and os.path.basename(f) != main.get_admin_name()):
-      dirs.append(f)
-    elif os.path.isfile(f):
-      files.append(f)
-
-  # add each file as a child of CURRENT_PARENT
-  for f in files:
-    fcontents = get_text(f)
-    if load_props:
-      fprops = get_props(f)
-    else:
-      fprops = {}
-    current_parent.add_child(SVNTreeNode(os.path.basename(f), None,
-                                         fcontents, fprops))
-
-  # for each subdir, create a node, walk its tree, add it as a child
-  for d in dirs:
-    if load_props:
-      dprops = get_props(d)
-    else:
-      dprops = {}
-    new_dir_node = SVNTreeNode(os.path.basename(d), None, None, dprops)
-    handle_dir(d, new_dir_node, load_props, ignore_svn)
-    current_parent.add_child(new_dir_node)
-
 def get_child(node, name):
   """If SVNTreeNode NODE contains a child named NAME, return child;
-  else, return None. If SVNTreeNode is not a directory, raise a
-  SVNTreeIsNotDirectory exception"""
+  else, return None. If SVNTreeNode is not a directory, exit completely."""
   if node.children == None:
-    raise SVNTreeIsNotDirectory
+    logger.error("Foolish call to get_child.")
+    sys.exit(1)
   for n in node.children:
-    if (name == n.name):
+    if name == n.name:
       return n
   return None
 
@@ -420,8 +588,8 @@ def get_child(node, name):
 def default_singleton_handler(node, description):
   """Print SVNTreeNode NODE's name, describing it with the string
   DESCRIPTION, then raise SVNTreeUnequal."""
-  print "Couldn't find node '%s' in %s tree" % (node.name, description)
-  node.pprint()
+  logger.warn("Couldn't find node '%s' in %s tree" % (node.name, description))
+  logger.warn(str(node))
   raise SVNTreeUnequal
 
 # A test helper function implementing the singleton_handler_a API.
@@ -438,10 +606,15 @@ def detect_conflict_files(node, extra_files):
       break
   else:
     msg = "Encountered unexpected disk path '" + node.name + "'"
-    print msg
-    node.pprint()
+    logger.warn(msg)
+    logger.warn(str(node))
     raise SVNTreeUnequal(msg)
 
+def detect_conflict_files_done(extra_files):
+   """Done handler for detect_conflict_files"""
+   if len(extra_files):
+    raise SVNTreeError("Not all extra reject files have been accounted for")
+
 ###########################################################################
 ###########################################################################
 # EXPORTED ROUTINES ARE BELOW
@@ -473,29 +646,32 @@ def compare_trees(label,
 
   def display_nodes(a, b):
     'Display two nodes, expected and actual.'
-    print "============================================================="
-    print "Expected '%s' and actual '%s' in %s tree are different!" \
-          % (b.name, a.name, label)
-    print "============================================================="
-    print "EXPECTED NODE TO BE:"
-    print "============================================================="
-    b.pprint()
-    print "============================================================="
-    print "ACTUAL NODE FOUND:"
-    print "============================================================="
-    a.pprint()
+    o = StringIO()
+    o.write("=============================================================\n")
+    o.write("Expected '%s' and actual '%s' in %s tree are different!\n"
+                % (b.name, a.name, label))
+    o.write("=============================================================\n")
+    o.write("EXPECTED NODE TO BE:\n")
+    o.write("=============================================================\n")
+    b.pprint(o)
+    o.write("=============================================================\n")
+    o.write("ACTUAL NODE FOUND:\n")
+    o.write("=============================================================\n")
+    a.pprint(o)
+    logger.warn(o.getvalue())
+    o.close()
 
   # Setup singleton handlers
-  if (singleton_handler_a is None):
+  if singleton_handler_a is None:
     singleton_handler_a = default_singleton_handler
     a_baton = "expected " + label
-  if (singleton_handler_b is None):
+  if singleton_handler_b is None:
     singleton_handler_b = default_singleton_handler
     b_baton = "actual " + label
 
   try:
     # A and B are both files.
-    if ((a.children is None) and (b.children is None)):
+    if (a.children is None) and (b.children is None):
       if compare_file_nodes(a, b):
         display_nodes(a, b)
         raise SVNTreeUnequal
@@ -506,8 +682,7 @@ def compare_trees(label,
       raise SVNTypeMismatch
     # They're both directories.
     else:
-      # First, compare the directories' two hashes.
-      if (a.props != b.props) or (a.atts != b.atts):
+      if compare_dir_nodes(a, b):
         display_nodes(a, b)
         raise SVNTreeUnequal
 
@@ -527,49 +702,80 @@ def compare_trees(label,
         else:
           singleton_handler_a(a_child, a_baton)
       for b_child in b.children:
-        if (b_child not in accounted_for):
+        if b_child not in accounted_for:
           singleton_handler_b(b_child, b_baton)
   except SVNTypeMismatch:
-    print 'Unequal Types: one Node is a file, the other is a directory'
+    logger.warn('Unequal Types: one Node is a file, the other is a directory')
     raise SVNTreeUnequal
-  except SVNTreeIsNotDirectory:
-    print "Error: Foolish call to get_child."
-    sys.exit(1)
   except IndexError:
-    print "Error: unequal number of children"
+    logger.warn("Error: unequal number of children")
     raise SVNTreeUnequal
   except SVNTreeUnequal:
     if a.name != root_node_name:
-      print "Unequal at node %s" % a.name
+      logger.warn("Unequal at node %s" % a.name)
     raise
 
 
 
 # Visually show a tree's structure
 
-def dump_tree(n,indent=""):
-  "Print out a nice representation of the tree's structure."
+def _dump_tree(n,indent="",stream=sys.stdout):
+  """Print out a nice representation of the structure of the tree in
+  the SVNTreeNode N. Prefix each line with the string INDENT."""
 
   # Code partially stolen from Dave Beazley
-  if n.children is None:
-    tmp_children = []
-  else:
-    tmp_children = n.children
+  tmp_children = sorted(n.children or [])
 
   if n.name == root_node_name:
-    print "%s%s" % (indent, "ROOT")
+    stream.write("%s%s\n" % (indent, "ROOT"))
   else:
-    print "%s%s" % (indent, n.name)
+    stream.write("%s%s\n" % (indent, n.name))
 
   indent = indent.replace("-", " ")
   indent = indent.replace("+", " ")
   for i in range(len(tmp_children)):
     c = tmp_children[i]
-    if i == len(tmp_children
-                )-1:
-      dump_tree(c,indent + "  +-- ")
+    if i == len(tmp_children)-1:
+      _dump_tree(c,indent + "  +-- ",stream)
     else:
-      dump_tree(c,indent + "  |-- ")
+      _dump_tree(c,indent + "  |-- ",stream)
+
+
+def dump_tree(n):
+    output = StringIO()
+    _dump_tree(n,stream=output)
+    logger.warn(output.getvalue())
+    output.close()
+
+
+def dump_tree_script__crawler(n, subtree="", stream=sys.stdout):
+  "Helper for dump_tree_script. See that comment."
+  count = 0
+
+  # skip printing the root node.
+  if n.name != root_node_name:
+    count += n.print_script(stream, subtree)
+
+  for child in n.children or []:
+    count += dump_tree_script__crawler(child, subtree, stream)
+
+  return count
+
+
+def dump_tree_script(n, subtree="", stream=sys.stdout, wc_varname='wc_dir'):
+  """Print out a python script representation of the structure of the tree
+  in the SVNTreeNode N. Print only those nodes whose path string starts
+  with the string SUBTREE, and print only the part of the path string
+  that remains after SUBTREE.
+  The result is printed to STREAM.
+  The WC_VARNAME is inserted in the svntest.wc.State(wc_dir,{}) call
+  that is printed out (this is used by factory.py)."""
+
+  stream.write("svntest.wc.State(" + wc_varname + ", {")
+  count = dump_tree_script__crawler(n, subtree, stream)
+  if count > 0:
+    stream.write('\n')
+  stream.write("})")
 
 
 ###################################################################
@@ -609,32 +815,12 @@ def build_generic_tree(nodelist):
 # Parse co/up output into a tree.
 #
 #   Tree nodes will contain no contents, a 'status' att, and a
-#   'writelocked' att.
+#   'treeconflict' att.
 
-def build_tree_from_checkout(lines, include_skipped=1):
+def build_tree_from_checkout(lines, include_skipped=True):
   "Return a tree derived by parsing the output LINES from 'co' or 'up'."
 
-  root = SVNTreeNode(root_node_name)
-  rm1 = re.compile ('^([RMAGCUDE_ ][MAGCUDE_ ])([B ])\s+(.+)')
-  if include_skipped:
-    rm2 = re.compile ('^(Restored|Skipped)\s+\'(.+)\'')
-  else:
-    rm2 = re.compile ('^(Restored)\s+\'(.+)\'')
-
-  for line in lines:
-    match = rm1.search(line)
-    if match and match.groups():
-      new_branch = create_from_path(match.group(3), None, {},
-                                    {'status' : match.group(1)})
-      root.add_child(new_branch)
-    else:
-      match = rm2.search(line)
-      if match and match.groups():
-        new_branch = create_from_path(match.group(2), None, {},
-                                      {'verb' : match.group(1)})
-        root.add_child(new_branch)
-
-  return root
+  return svntest.wc.State.from_checkout(lines, include_skipped).old_tree()
 
 
 # Parse ci/im output into a tree.
@@ -644,21 +830,7 @@ def build_tree_from_checkout(lines, include_skipped=1):
 def build_tree_from_commit(lines):
   "Return a tree derived by parsing the output LINES from 'ci' or 'im'."
 
-  # Lines typically have a verb followed by whitespace then a path.
-  root = SVNTreeNode(root_node_name)
-  rm1 = re.compile ('^(\w+(  \(bin\))?)\s+(.+)')
-  rm2 = re.compile ('^Transmitting')
-
-  for line in lines:
-    match = rm2.search(line)
-    if not match:
-      match = rm1.search(line)
-      if match and match.groups():
-        new_branch = create_from_path(match.group(3), None, {},
-                                      {'verb' : match.group(1)})
-        root.add_child(new_branch)
-
-  return root
+  return svntest.wc.State.from_commit(lines).old_tree()
 
 
 # Parse status output into a tree.
@@ -666,110 +838,29 @@ def build_tree_from_commit(lines):
 #   Tree nodes will contain no contents, and these atts:
 #
 #          'status', 'wc_rev',
-#             ... and possibly 'locked', 'copied', 'writelocked',
+#             ... and possibly 'locked', 'copied', 'switched',
+#             'writelocked' and 'treeconflict',
 #             IFF columns non-empty.
 #
 
 def build_tree_from_status(lines):
   "Return a tree derived by parsing the output LINES from 'st -vuq'."
 
-  root = SVNTreeNode(root_node_name)
-
-  # 'status -v' output looks like this:
-  #
-  #      "%c%c%c%c%c%c %c   %6s   %6s %-12s %s\n"
-  #
-  # (Taken from 'print_status' in subversion/svn/status.c.)
-  #
-  # Here are the parameters.  The middle number in parens is the
-  # match.group(), followed by a brief description of the field:
-  #
-  #    - text status           (1)  (single letter)
-  #    - prop status           (1)  (single letter)
-  #    - wc-lockedness flag    (2)  (single letter: "L" or " ")
-  #    - copied flag           (3)  (single letter: "+" or " ")
-  #    - switched flag         (4)  (single letter: "S" or " ")
-  #    - repos lock status     (5)  (single letter: "K", "O", "B", "T", " ")
-  #
-  #    [one space]
-  #
-  #    - out-of-date flag      (6)  (single letter: "*" or " ")
-  #
-  #    [three spaces]
-  #
-  #    - working revision      (7)  (either digits or "-")
-  #
-  #    [one space]
-  #
-  #    - last-changed revision (8)  (either digits or "?")
-  #
-  #    [one space]
-  #
-  #    - last author           (9)  (string of non-whitespace characters)
-  #
-  #    [one space]
-  #
-  #    - path                 (10)  (string of characters until newline)
-
-  # Try http://www.wordsmith.org/anagram/anagram.cgi?anagram=ACDRMGU
-  rm = re.compile('^([!MACDRUG_ ][MACDRUG_ ])([L ])([+ ])([S ])([KOBT ]) ([* ])   [^0-9-]*(\d+|-|\?) +(\d|-|\?)+ +(\S+) +(.+)')
-  for line in lines:
-
-    # Quit when we hit an externals status announcement (### someday we can fix
-    # the externals tests to expect the additional flood of externals status
-    # data).
-    if re.match(r'^Performing', line):
-      break
-
-    match = rm.search(line)
-    if match and match.groups():
-      if match.group(9) != '-': # ignore items that only exist on repos
-        atthash = {'status' : match.group(1),
-                   'wc_rev' : match.group(7)}
-        if match.group(2) != ' ':
-          atthash['locked'] = match.group(2)
-        if match.group(3) != ' ':
-          atthash['copied'] = match.group(3)
-        if match.group(4) != ' ':
-          atthash['switched'] = match.group(4)
-        if match.group(5) != ' ':
-          atthash['writelocked'] = match.group(5)
-        new_branch = create_from_path(match.group(10), None, {}, atthash)
-
-      root.add_child(new_branch)
-
-  return root
+  return svntest.wc.State.from_status(lines).old_tree()
 
 
 # Parse merge "skipped" output
 
 def build_tree_from_skipped(lines):
 
-  root = SVNTreeNode(root_node_name)
-  ### Will get confused by spaces in the filename
-  rm = re.compile ("^Skipped.* '([^ ]+)'\n")
+  return svntest.wc.State.from_skipped(lines).old_tree()
 
-  for line in lines:
-    match = rm.search(line)
-    if match and match.groups():
-      new_branch = create_from_path(match.group(1))
-      root.add_child(new_branch)
-
-  return root
 
 def build_tree_from_diff_summarize(lines):
   "Build a tree from output of diff --summarize"
-  root = SVNTreeNode(root_node_name)
-  rm = re.compile ("^([MAD ][M ])     (.+)\n")
 
-  for line in lines:
-    match = rm.search(line)
-    if match and match.groups():
-      new_branch = create_from_path(match.group(2),
-                                    atts={'status': match.group(1)})
-      root.add_child(new_branch)
+  return svntest.wc.State.from_summarize(lines).old_tree()
 
-  return root
 
 ####################################################################
 # Build trees by looking at the working copy
@@ -786,18 +877,4 @@ def build_tree_from_wc(wc_path, load_props=0, ignore_svn=1):
     files.  If IGNORE_SVN is true, then exclude SVN admin dirs from the tree.
     If LOAD_PROPS is true, the props will be added to the tree."""
 
-    root = SVNTreeNode(root_node_name, None)
-
-    # if necessary, store the root dir's props in a new child node '.'.
-    if load_props:
-      props = get_props(wc_path)
-      if props:
-        root_dir_node = SVNTreeNode(os.path.basename('.'), None, None, props)
-        root.add_child(root_dir_node)
-
-    # Walk the tree recursively
-    handle_dir(os.path.normpath(wc_path), root, load_props, ignore_svn)
-
-    return root
-
-### End of file.
+    return svntest.wc.State.from_wc(wc_path, load_props, ignore_svn).old_tree()
